/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIAddonInterposition.idl
 */

#ifndef __gen_nsIAddonInterposition_h__
#define __gen_nsIAddonInterposition_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#include "js/Value.h"

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIAddonInterposition */
#define NS_IADDONINTERPOSITION_IID_STR "e5453950-d95a-11e3-9c1a-0800200c9a66"

#define NS_IADDONINTERPOSITION_IID \
  {0xe5453950, 0xd95a, 0x11e3, \
    { 0x9c, 0x1a, 0x08, 0x00, 0x20, 0x0c, 0x9a, 0x66 }}

class NS_NO_VTABLE nsIAddonInterposition : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IADDONINTERPOSITION_IID)

  /* jsval interpose (in jsval addonId, in jsval target, in nsIIDPtr iface, in jsval prop); */
  NS_IMETHOD Interpose(JS::HandleValue addonId, JS::HandleValue target, const nsIID *iface, JS::HandleValue prop, JS::MutableHandleValue _retval) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIAddonInterposition, NS_IADDONINTERPOSITION_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIADDONINTERPOSITION \
  NS_IMETHOD Interpose(JS::HandleValue addonId, JS::HandleValue target, const nsIID *iface, JS::HandleValue prop, JS::MutableHandleValue _retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIADDONINTERPOSITION(_to) \
  NS_IMETHOD Interpose(JS::HandleValue addonId, JS::HandleValue target, const nsIID *iface, JS::HandleValue prop, JS::MutableHandleValue _retval) { return _to Interpose(addonId, target, iface, prop, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIADDONINTERPOSITION(_to) \
  NS_IMETHOD Interpose(JS::HandleValue addonId, JS::HandleValue target, const nsIID *iface, JS::HandleValue prop, JS::MutableHandleValue _retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->Interpose(addonId, target, iface, prop, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsAddonInterposition : public nsIAddonInterposition
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIADDONINTERPOSITION

  nsAddonInterposition();

private:
  ~nsAddonInterposition();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsAddonInterposition, nsIAddonInterposition)

nsAddonInterposition::nsAddonInterposition()
{
  /* member initializers and constructor code */
}

nsAddonInterposition::~nsAddonInterposition()
{
  /* destructor code */
}

/* jsval interpose (in jsval addonId, in jsval target, in nsIIDPtr iface, in jsval prop); */
NS_IMETHODIMP nsAddonInterposition::Interpose(JS::HandleValue addonId, JS::HandleValue target, const nsIID *iface, JS::HandleValue prop, JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIAddonInterposition_h__ */
