/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[ptr] native RequestHeaderTuples(mozilla::net::RequestHeaderTuples);

[uuid(306ACF4D-C6DF-4EF6-BDA9-5CB92E83EDD9)]
interface nsIHttpChannelChild : nsISupports
{
  void addCookiesToRequest();

  // Headers that the channel client has set via SetRequestHeader.
  readonly attribute RequestHeaderTuples clientSetRequestHeaders;
};
